/*
 * @(#)context_md.h	1.21 97/02/25
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

/*
 * Freebsd-dependent API
 */

#ifndef _FREEBSD_CONTEXT_MD_H_
#define _FREEBSD_CONTEXT_MD_H_

#include <errno.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/signal.h>
#include <setjmp.h>

typedef struct lj_ucontext {
	sigjmp_buf      jmpbuf;
	char            floatbuf[108];
} lj_ucontext_t;

/*
 * Routines that call getcontext have strange control flow graphs,
 * since a call to a routine that calls setcontext will eventually
 * return at the getcontext site, not the original call site.  This
 * utterly wrecks control flow analysis.
 */
#pragma unknown_control_flow(getcontext)

/*
 * A freebsd2 context.  Everything else is on the stack.
 */
typedef struct {
    unsigned int unix_errno;
    lj_ucontext_t lj_ucontext;
    sigset_t mask;
} context_t;

/* Save state of the current content and call rescheduler */

void reschedule(void);

#if defined(i386)

#define	switchContext(contextp) {			\
    errno = (contextp)->unix_errno;			\
    siglongjmp((contextp)->lj_ucontext.jmpbuf,1);	\
}

#ifdef SIGIO_HACK
  #define __saveMask(ctx)    sigprocmask(0, 0, &(ctx)->mask)
  #define __restMask(ctx)    sigprocmask(SIG_SETMASK, &(ctx)->mask, 0)
#else
  #define __saveMask(ctx)    do {} while (0)
  #define __restMask(ctx)    do {} while (0)
#endif

#define yieldContext(contextp) {				\
    char * fdata = (char *)contextp->lj_ucontext.floatbuf;	\
    __asm__ ("fsave %0"::"m" (*fdata));				\
    __saveMask(contextp);					\
    __asm__ ("fwait");						\
    if (!sigsetjmp(contextp->lj_ucontext.jmpbuf,-1)) {		\
          (contextp)->unix_errno = errno;			\
          reschedule();						\
    } else {							\
	__asm__ ("frstor %0"::"m" (*fdata));			\
	__restMask(contextp);					\
	__asm__ ("fwait");					\
    }								\
}
#else
#error Unsupported ISA
#endif
#endif /* !_FREEBSD_CONTEXT_MD_H_ */
